;tabs set at 4 spaces

#include "keys.inc"
#include "ion.inc"
#include "ti83plus.inc"

.org    progstart-2			;9D95h - 2, so 9D93h
.db     $BB,$6D				;AsmPrgm token
	ret						;if it's run from the homescreen, exit
	 jr nc,setup			;where to go to start the program
title_txt:
.db 10,"prgmBAD",10,0				;the title

;declare all the variables - these don't add any actual code, they are just definitions
lines = saferam1			;the array of lines (13 bytes)
x = saferam1+13				;player's x position
y = saferam1+14				;current y position of the falling lines
menu = saferam1+15			;current menu choice
score = saferam1+16			;your score

counter = saferam1+18		;interrupt counter
link_port	= saferam1+19	;current state of link port (lines up/down)
tone_ctr	= saferam1+20	;note's tone
length_ctr	= saferam1+21	;how long the note should be
cur_note	= saferam1+22	;position in song
rest		= saferam1+23	;pause between notes so they don't run together

setup:						;preparatory stuff befor we start the main menu
	set textWrite,(iy+sGrFlags)	;text on the buffer 
	xor	a					;store seventeen to a
	ld (menu),a				;17 to x
	bcall(_ClrLCDFull)		;clear the screen

	di
	ld a,$D0				;default link port value
	out (0),a

	xor a					;set interrupts at fastest speed
	out (4),a

	ld a,$99
	ld bc,$0100
	ld h,a
	ld d,a
	ld l,c
	ld e,b
	ld i,a
	inc a
	ld (hl),a
	ldir
	ld l,a
	ld (hl),$c3				;jp opcode
	inc l
	ld (hl),interrupt & $ff
	inc l
	ld (hl),interrupt >> 8
	
start:                      ;label start
	xor a					;reset song
	ld (cur_note),a
	ld (rest),a

	ld a,$D0
	ld (link_port),a
	ld hl,music
	ld de,tone_ctr
	ldi
	ldi

	call dispTitle			;routine that displays the title
	ld b,3
	ld hl,menu1
	call dispMenu
;draw high scores
	ld hl,33*256
	ld (penCol),hl
	ld hl,high1
	bcall(_VPutS)
	ld hl,penCol
	inc (hl)
	ld hl,player1
	bcall(_VPutS)
	ld hl,dash
	bcall(_VPutS)
	ld hl,(highscore)
	call dispVar+3			;skip the "ld (penCol),de" (3-byte instruction, hence the +3)

	ld hl,40*256
	ld (penCol),hl
	ld hl,high2
	bcall(_VPutS)
	ld hl,penCol
	inc (hl)
	ld hl,player2
	bcall(_VPutS)
	ld hl,dash
	bcall(_VPutS)
	ld hl,(highscore+2)
	call dispVar+3			;skip the "ld (penCol),de" (3-byte instruction, hence the +3)

start_loop:
	call dispBar			;display the bar according to menu selection
	call ionFastCopy		;copy the graph buffer to the screen
	call dispBar			;erase the bar
;;;;menu keys;;;;
	bcall(_GetCSC)			;get a key press
	cp skEnter				;was enter pressed?
	 jp z,second			;..
	cp sk2nd				;was 2nd pressed?
	 jp z,second			;..
	cp skMode				;was mode pressed?
	 ret z					;.. quit
	cp skClear				;was mode pressed?
	 ret z					;.. quit
	ld hl,start_loop
	push hl
	cp skUp					;Was it the up key?
	 jp z,up				;.. then move bar up
	cp skDown				;was it the down key?
	 jp z,down				;.. then move bar down
	ret						;repeat until an action has taken place

;;;;play the game;;;;
play:
	di
	bcall(_ClrLCDFull)		;clear the screen
	bcall(_GrBufClr)		;clear the graph buffer
	ld hl,$0000
	ld (score),hl			;reset score to 0
	call addScore			;set up the lines that will fall from the top
	ld a,45					;45 is the player's x position
	ld (x),a				;
	xor a
	ld (y),a
	ld hl,counter
	ld (hl),a
	im 2
	ei						;start interrupt
play_loop:					;main gameplay loop
	call drawLines
	call drawShip
	call ionFastCopy
	call drawShip
;check keys
	ld a,Group1				;Enable group 1. ($FE)
	call get_key
	cp kdRight				;if the right arrow was pressed
	 call z,shipRight		;.. move ship to the right
	cp kdLeft				;if the left arrow was pressed
	 call z,shipLeft		;.. move ship to the left
	ld a,Group7 & Group2	;Enable group 2 ($FD) and group 7 ($BF)
	call get_key
	cp kdMode				;[Mode] and [Clear] return the same values
	 jp z,lose				;.. so this checks for both

;check collision
	ld hl,gbuf+(12*63)
	ld a,(x)
	rra
	rra
	rra
	and $0F
	ld e,a
	ld d,0
	add hl,de
	ld a,(x)
	and $7
	ld b,a
	ld a,%11100000			;collision box
	 jr z,skip
		rra
		rr d
	djnz $-3
skip:
	and (hl)
	 jp nz,lose
	inc hl
	ld a,d
	and (hl)
	 jp nz,lose
;end collision check
	jr play_loop

shipRight:
	ld hl,x
	ld a,(hl)
	cp 93
	 ret z
	push hl
		add a,3
		ld e,63
		call ionGetPixel
		and (hl)
	pop hl
	 ret nz
	inc (hl)
	ret

shipLeft:
	ld hl,x
	ld a,(hl)
	or a					;check if a = 0 (you can also use "and a")
	 ret z
	push hl
		dec a
		ld e,63
		call ionGetPixel
		and (hl)
	pop hl
	 ret nz
	dec (hl)
	ret

drawLines:
	ld a,(speed)
	add a,a
	add a,a
	ld b,a
	ld hl,counter
	ld a,(hl)
	sub b
	 ret c
	ld (hl),a

	ld a,(gameMode)
	or a
	ld a,$B6						;opcode for or (hl)
	 jr z,$+4
		ld a,$AE					;opcode for xor (hl)
	ld (or_xor),a

	ld hl,lines
	ld bc,(numLines-1)				;c = numLines-1 (it's just junk), b = numLines
dL_loop:
	push bc
	push hl
		ld de,(y)                   ;load the variable y (the y-location of the current line)
		ld a,(hl)
		call ionGetPixel
or_xor = $
		xor (hl)
		ld (hl),a
	pop hl
	pop bc
	inc hl
	djnz dL_loop

	ld hl,y
	inc (hl)

	ld a,(gameMode)			;check which game mode (0 or 1) we're using
	add a,a					;mode 1 = 1*2+62 = last pixel down to 64 (bottom of screen)
	add a,62				;mode 0 = 0*2+62 = last pixel 62 (two pixels up from bottom of screen)
	ld b,a
	ld a,(hl)
	cp b
	 jp z,loadLines
	ret

drawShip:
	ld l,62
	ld a,(x)
	ld b,2
	ld ix,ship
	jp ionPutSprite

loadLines:
	ld a,(gameMode)
	or a
	 jr nz,addScore			;if game mode = 0, we need to check if there has been a collision
	ld hl,lines
	ld bc,(numLines)
	ld b,a
	ld a,(x)
	ld e,3
line_check:
	push hl
	push bc
	cpir
	pop bc
	pop hl
	 jr nz,$+6
		 pop hl				;remember that drawLines was called
		 jp lose			;.. so we need to clear the stack
	inc a
	dec e
	 jr nz,line_check
addScore:
;increase score
	ld hl,speed
	ld a,50
	sub (hl)				;a = 50-speed
	ld hl,(score)
	ld de,(numLines)
	ld d,0
	add hl,de
	add hl,de				;score += numLines*2
	ld e,a
	add hl,de				;score = numeLines*2 + (50-speed)
	ld (score),hl			;save score
;now reload the lines
	ld hl,lines
	ld a,(numLines)
lL_loop:
	push af
		push hl
			ld b,94
			call ionRandom
		pop hl
		ld (hl),a
		inc hl
	pop af
	dec a
	 jr nz,lL_loop
	ld (y),a
	ret

;update group in key port and read
get_key:
	out (1),a				;changes the group of keys to read from (see [keys.inc] for list of groups)
	push af
	pop af
	in a,(1)				;read byte form port 1 (key port) and store in a
	ret

;;;;other stuff;;;;
dispTitle:					;this is the routine that displays the title at the top.
	bcall(_GrBufClr)		;clear the graph buffer
	set fracDrawLFont, (iy+fontFlags)	;use large font with _VPutS
	ld hl,title_txt
	ld de,0
	ld (penCol),de
	bcall(_VPutS)
	res fracDrawLFont, (iy+fontFlags)
	ld hl,RevSoft_txt
	ld d,56
	ld (penCol),de
	bcall(_VPutS)
	ret

options:                    ;this is the options screen
	xor a
	ld (menu),a
options_loop:
	call dispTitle             ;display the title
	ld b,3
	ld hl,menu4
	call dispMenu

	ld de,$0938				;load 17 (the numbers are in hex) into h and 76 into l
	ld hl,(numLines)		;load the variable speed into l (because it is one byte)
	ld h,0
	call dispVar			;call the label dispVar (to display it at the coordinates
                            ;in (pencol) and (penrow))
	ld de,$0F38
	ld hl,(gameMode)
	ld h,0
	inc l
	call dispVar			;display current game mode

	ld de,$1538				;text coordinates
	ld hl,(speed)			;load the variable speed into l (because it is one byte)
	ld h,0
	call dispVar			;display the variable
	call dispBar			;display the bar that highlites the text
	call ionFastCopy		;display the buffer

;;;;menu keys;;;;
		bcall(_GetCSC)		;check if a key was pressed
		or a
	 jr z,$-4

	cp skMode				;was it the [Mode] key?
	 jp z,start				;.. if so return to the main menu
	cp skClear				;was it the [Clear] key?
	 jr nz,$+10
		ld a,1
		ld (menu),a
		jp start
;--- ret returns us to options_loop
	ld hl,options_loop
	push hl
;---
	cp skRight				;was it the right key?
	 jr z,right				;
	cp skLeft				;was it the left key?
	 jr z,left				;
	cp skUp					;was it the up key?
	 jp z,up				;
	cp skDown				;was it the down key?
	 jp z,down				;
	ret						;go to label options

right:
	ld a,(menu)
	or a
	 jr nz,right2
		ld hl,numLines
		ld a,(hl)
		cp 13				;max 13 lines at a time
		 ret z
		inc (hl)			;increase numLines by one
		ret
right2:
	dec a
	 jr nz,right3
	 	inc a
		ld (gameMode),a		;load 1 into gameMode
		ret
right3:
	ld hl,speed
	ld a,(hl)
	cp 10
	 ret z
	inc (hl)
	ret

left:
	ld a,(menu)
	or a
	 jr nz,left2
		ld hl,numLines
		ld a,(hl)
		cp 3				;need at least 3 lines at a time
		 ret z
		dec (hl)
		ret
left2:
	dec a
	 jr nz,left3
		ld (gameMode),a		;a = zero because of "dec a"
		ret
left3:
	ld hl,speed
	ld a,(hl)
	dec a
	 ret z
	dec (hl)
	ret

;;;;sub routines;;;;
dispBar:
	ld a,(menu)
	ld c,a
	rrca					;*128
	rrca					;*64
	ld b,a					;save a*64 in b
	ld a,c
	add a,a					;*2
	add a,a					;*4
	add a,a					;*8
	add a,b					;a*8 + a*64 = a*72
	ld hl,gbuf+(12*09)
	ld c,a
	ld b,0
	add hl,bc				;now we have the offset in gbuf
	ld c,6					;6 rows
invert_out:
	ld b,8					;7 columns (remember each byte is 8 pixels, so bar is 7*8 pixels wide x 6 rows)
invert_in:
	ld a,(hl)
	cpl						;cpl is a faster/smaller xor $FF
	ld (hl),a
	inc hl
	djnz invert_in
	ld de,4
	add hl,de				;move to the next row (we've drawn 8, there are 12 columns, 12-8 = 4)
	dec c					;c is our row counter
	 jr nz,invert_out
	ret

dispVar:
	ld (penCol),de
	bcall(_SetXXXXOP2)         ;load hl into op2 (11-bit register)
	bcall(_OP2ToOP1)           ;load op2 into op1
	ld a,5                     ;load 5 into a (a is the maximum amount of numbers to display)
	bcall(_DispOP1A)           ;display op1
	ret

;;;;menu subs;;;;
up:
	ld hl,menu
	ld a,(hl)
	or a
	 ret z
	dec (hl)
	ret

down:
	ld hl,menu
	ld a,(hl)
	cp 2
	 ret z
	inc (hl)
	ret

second:
	ld a,(menu)
	or a
	 jp z,play
 	dec a
	 jp z,options
	ret

;hl = start of menu
;b = # of entries
dispMenu:
	ld de,0900h
dM_loop:
	ld (penCol),de		;penCol = e, penCol+1 (aka penRow) = d
	bcall(_VPutS)
	ld a,d
	add a,6
	ld d,a
	djnz dM_loop
	ret

;you lost
lose:
	im 1				;reset interrupt mode to normal OS interrupt


	bcall(_HomeUp)
	ld hl,losetxt
	bcall(_PutS)

	bcall(_NewLine)
	ld hl,scoretxt
	bcall(_PutS)

	ld hl,(score)
	bcall(_DispHL)
	call crash
lose_loop:
	bcall(_GetCSC)
	or a
	 jr z,lose_loop
	ld hl,(score)
	ld de,(highscore)
	ld a,(gameMode)
	or a
	 jr z,$+5
		ld de,(highscore+2)
	sbc hl,de			;check if score > highscore
	 jp c,start			;c means it's not
;you got a high score!
high:
	add hl,de			;save the high score (we subtracted de, now we add it back on)
	or a
	 jr z,$+7
		ld (highscore+2),hl
		jr $+5
	ld (highscore),hl	;save to appropriate high score bytes
	bcall(_HomeUp)
	bcall(_ClrLCDFull)
	ld hl,highscoretxt
	bcall(_PutS)
	ld hl,$0002
	ld (curRow),hl
	ld hl,enternametxt
	bcall(_PutS)
	ld a,(gameMode)
	or a
	ld hl,player1
	 jr z,$+5
		ld hl,player2
	ld de,$0003
	call input_name
	jp start

;our interrupt, a simple counter for the speed of the lines
interrupt:
	di
	exx				;swap bc, de, hl with their shadow registers
	ex	af,af'		;swap af with it's shadow register (af')

	ld hl,counter
	inc (hl)

;music

	ld hl,rest
	ld a,(hl)
	or a
	 jr z,$+5
		dec (hl)
		jr no_out
	
	ld hl,tone_ctr
	dec (hl)
	 jr nz,no_out

	ld a,(cur_note)
	add a,a				;x2, each note takes up 2 bytes (tone, length)
	ld l,a
	ld h,0
	ld de,music
	add hl,de			;find current note that we're playing
	ld a,(hl)
	ld (tone_ctr),a		;reset tone_ctr
	ld a,(link_port)
	xor %0011		;alternate link port from low to high (that's what produces the sound)
	ld (link_port),a
	out (0),a

	ld hl,length_ctr
	dec (hl)
	 jr nz,no_out		;if note isn't finished yet, don't load next note

	ld hl,cur_note
	inc (hl)
	ld a,(hl)
	cp (music_end-music)/2
	 jr nz,$+4
		xor a
		ld (hl),a
	add a,a
	ld l,a
	ld h,0
	ld de,music
	add hl,de			;find current note that we're playing
	ld a,(hl)			;note tone
	ld (tone_ctr),a
	inc hl
	ld a,(hl)			;length of note
	ld (length_ctr),a
	ld a,3
	ld (rest),a
no_out:

;end interrupt
	ld  a,%1000		;acknowledge interrupt
	out (3),a
	ld  a,%1010		;reset interrupt mask
	out (3),a
	ex	af,af'		;swap the original af back
	exx				;swap the original bc, de, hl back
	ei				;reenable interrupts
   	ret				;return from interrupt


crash:
	ld de,60
crash_outer:
	ld hl,100
crash_wait:
	dec hl
	push hl
	pop af
	or l
	 jr nz,crash_wait
	ld a,(link_port)
	xor %0011		;alternate link port from low to high (that's what produces the sound)
	ld (link_port),a
	out (0),a
	ld a,r
	ld b,a
	ld a,(crash_outer+2)
	add a,b
	add a,a
	ld b,a
	ld a,r
	add a,b
	and $F
	ld (crash_outer+2),a
	dec de
	ld a,e
	or d
	 jr nz,crash_outer
	ret
	

;hl = string to store to
;de = coordinates
input_name:
	ld (currow_smc),de
	ld (text_smc),hl
	call clear_string
input_loop:
currow_smc = $+1
	ld de,$0000
	ld (curRow),de
text_smc = $+1
	ld hl,$0000
	bcall(_PutS)
	ld a,' '
	bcall(_PutC)
;draw cursor
	ld de,(currow_smc)
	ld a,(cursor)
	ld d,a
	ld (curRow),de
	cp 11
cursor_smc = $+1
	ld a,$E2
	 jr nz,$+4
		ld a,$F1
	bcall(_PutC)
key_loop:
	bcall(_GetCSC)
	ld d,0
;first check special keys
	cp skAlpha
	 jr nz,$+12
		ld a,(cursor_smc)
		xor 1
		ld (cursor_smc),a
		jr input_loop
	cp skClear
	 call z,clear_string
	 jr z,input_loop
	cp skEnter
	 jr nz,skipEnter
		ld a,(cursor)
		or a
		 jr z,key_loop
	 	ld hl,(text_smc)
		ld e,a
		ld d,0
		add hl,de
		ld (hl),0			;add terminating 0
		ret
skipEnter:
	cp skDel
	 jr nz,skipDelete
		ld hl,cursor
		ld a,(hl)
		or a
		 jr z,$+3
		 	dec a
		ld (hl),a
		ld e,a
		ld hl,(text_smc)
		add hl,de
		ld (hl),' '
		jr input_loop
skipDelete:
;now letter keys
	sub $B
	 jr c,key_loop
	cp 37					;the last acceptable key is [Math]
	 jr nc,key_loop
	ld hl,char_list
	ld e,a
	add hl,de
	ld a,(hl)
	cp 20
	 jr c,input_loop
	ld hl,(text_smc)
cursor = $+1
	ld e,$00
	add hl,de
	ld c,a					;save character value
	ld a,e					;check cursor value, can only draw 11 characters
	cp 11
	 jp z,input_loop
	push af
		ld a,(cursor_smc)	;if small alpha is set, use lowercase letters
		rra
		 jr nc,$+6
			ld a,$20
			add a,c
			ld c,a
	pop af
	ld (hl),c
	inc a
	ld (cursor),a
	jp input_loop

clear_string:
	xor a
	ld (cursor),a
	ld hl,cursor_smc
	res 0,(hl)
	ld hl,(text_smc)
	ld (hl),' '
	ld e,l
	ld d,h
	inc de
	ld bc,10
	ldir
	ret

;corresponds to _GetCSC values
char_list:
.db "WRMH  "
.db "?",$5B,"VQLG  "
.db ":ZUPKFC "
.db " YTOJEBX "
.db "XSNIDA"

;the song that plays in the background:
music:
.db 2,30
.db 4,20
.db 2,30
.db 4,20
.db 2,50
.db 3,51
.db 5,52
.db 4,50

.db 2,30
.db 4,20
.db 2,30
.db 4,20
.db 1,80
.db 3,51
.db 5,52
.db 4,50

.db 2,30
.db 4,20
.db 2,30
.db 4,20
.db 2,50
.db 3,51
.db 5,52
.db $FF,1
.db 4,50

.db 2,40
.db 3,30
.db 6,11
.db 4,30
.db 5,30
.db 4,30
.db 5,30
.db 3,50
.db 4,51
.db 5,52
.db 4,50
.db 2,10
.db 3,50
.db 6,11
.db 2,40
.db 3,30
.db 6,11
.db 4,30
.db 5,30
.db 4,30
.db 5,30
.db 3,50
.db 4,51
.db 5,52
.db 4,50
.db 2,10
.db 3,50
.db 6,11
.db 2,30
.db 4,30
.db 2,30
.db 4,30

.db 1,100
.db 1,50
.db 1,100

.db $A0,1

.db 4,20
.db 3,40
.db 5,20
.db 3,40
.db 4,20
.db 3,40
.db 2,80

.db 5,15
.db 4,30
.db 6,15
.db 4,30
.db 5,15
.db 4,30

.db $40,1

.db $1,10
.db $2,9
.db $3,8
.db $4,8
.db $5,7
.db $6,7
.db $7,6
.db $8,6
.db $9,5
.db $10,4
.db $11,3

.db $20,3

.db 7,8
.db 6,10
.db 5,11
.db 4,12
.db 3,15
.db 2,20
.db 1,30
.db $70,1
music_end:

;;;;Variables that are saved;;;;
speed:		.db 3
gameMode:	.db 0
numLines:	.db 5
highscore:	.dw 0,0
player1:
.db "chickendude",0
player2:
.db "RevSoft",0,"   ",0

;;;;Text;;;;
dash:  .db " - ",0
menu1: .db "New game",0
menu2: .db "Options",0
menu3: .db "Quit",0

menu4: .db "Lines at a time:",0
menu5: .db "Gameplay mode:",0
menu6: .db "Speed:",0

high1: .db "Mode 1:",0
high2: .db "Mode 2:",0
losetxt: .db "You lose :(",0
scoretxt: .db "SCORE:",0
highscoretxt:
.db "You got a high  "
.db "score!",0
enternametxt:
.db "Enter your name:",0
RevSoft_txt:
.db "RevolutionSoftware.org",0
;;;;Sprites;;;;
ship:
.db %01000000
.db %11100000
